@@ -6,10 +6,13 @@ import logging
 import os
 import shutil
 import sys
+import sysconfig
 import tempfile
 from io import BytesIO
 
 import pytest
+from packaging.version import parse as parse_version
+
 from PIL import Image, ImageMath, features
 
 logger = logging.getLogger(__name__)
@@ -66,37 +69,31 @@ def convert_to_comparable(a, b):
 
 def assert_deep_equal(a, b, msg=None):
     try:
-        assert len(a) == len(b), msg or "got length {}, expected {}".format(
-            len(a), len(b)
-        )
+        assert len(a) == len(b), msg or f"got length {len(a)}, expected {len(b)}"
     except Exception:
         assert a == b, msg
 
 
 def assert_image(im, mode, size, msg=None):
     if mode is not None:
-        assert im.mode == mode, msg or "got mode {!r}, expected {!r}".format(
-            im.mode, mode
+        assert im.mode == mode, (
+            msg or f"got mode {repr(im.mode)}, expected {repr(mode)}"
         )
 
     if size is not None:
-        assert im.size == size, msg or "got size {!r}, expected {!r}".format(
-            im.size, size
+        assert im.size == size, (
+            msg or f"got size {repr(im.size)}, expected {repr(size)}"
         )
 
 
 def assert_image_equal(a, b, msg=None):
-    assert a.mode == b.mode, msg or "got mode {!r}, expected {!r}".format(
-        a.mode, b.mode
-    )
-    assert a.size == b.size, msg or "got size {!r}, expected {!r}".format(
-        a.size, b.size
-    )
+    assert a.mode == b.mode, msg or f"got mode {repr(a.mode)}, expected {repr(b.mode)}"
+    assert a.size == b.size, msg or f"got size {repr(a.size)}, expected {repr(b.size)}"
     if a.tobytes() != b.tobytes():
         if HAS_UPLOADER:
             try:
                 url = test_image_results.upload(a, b)
-                logger.error("Url for test images: %s" % url)
+                logger.error(f"Url for test images: {url}")
             except Exception:
                 pass
 
@@ -111,12 +108,8 @@ def assert_image_equal_tofile(a, filename, msg=None, mode=None):
 
 
 def assert_image_similar(a, b, epsilon, msg=None):
-    assert a.mode == b.mode, msg or "got mode {!r}, expected {!r}".format(
-        a.mode, b.mode
-    )
-    assert a.size == b.size, msg or "got size {!r}, expected {!r}".format(
-        a.size, b.size
-    )
+    assert a.mode == b.mode, msg or f"got mode {repr(a.mode)}, expected {repr(b.mode)}"
+    assert a.size == b.size, msg or f"got size {repr(a.size)}, expected {repr(b.size)}"
 
     a, b = convert_to_comparable(a, b)
 
@@ -128,13 +121,14 @@ def assert_image_similar(a, b, epsilon, msg=None):
     ave_diff = diff / (a.size[0] * a.size[1])
     try:
         assert epsilon >= ave_diff, (
-            msg or ""
-        ) + " average pixel value difference %.4f > epsilon %.4f" % (ave_diff, epsilon)
+            (msg or "")
+            + f" average pixel value difference {ave_diff:.4f} > epsilon {epsilon:.4f}"
+        )
     except Exception as e:
         if HAS_UPLOADER:
             try:
                 url = test_image_results.upload(a, b)
-                logger.error("Url for test images: %s" % url)
+                logger.error(f"Url for test images: {url}")
             except Exception:
                 pass
         raise e
@@ -164,17 +158,21 @@ def assert_tuple_approx_equal(actuals, targets, threshold, msg):
     assert value, msg + ": " + repr(actuals) + " != " + repr(targets)
 
 
-def skip_known_bad_test(msg=None):
-    # Skip if PILLOW_RUN_KNOWN_BAD is not true in the environment.
-    if not os.environ.get("PILLOW_RUN_KNOWN_BAD", False):
-        pytest.skip(msg or "Known bad test")
-
-
 def skip_unless_feature(feature):
-    reason = "%s not available" % feature
+    reason = f"{feature} not available"
     return pytest.mark.skipif(not features.check(feature), reason=reason)
 
 
+def skip_unless_feature_version(feature, version_required, reason=None):
+    if not features.check(feature):
+        return pytest.mark.skip(f"{feature} not available")
+    if reason is None:
+        reason = f"{feature} is older than {version_required}"
+    version_required = parse_version(version_required)
+    version_available = parse_version(features.version(feature))
+    return pytest.mark.skipif(version_available < version_required, reason=reason)
+
+
 @pytest.mark.skipif(sys.platform.startswith("win32"), reason="Requires Unix or macOS")
 class PillowLeakTestCase:
     # requires unix/macOS
@@ -189,7 +187,7 @@ class PillowLeakTestCase:
         :returns: memory usage in kilobytes
         """
 
-        from resource import getrusage, RUSAGE_SELF
+        from resource import RUSAGE_SELF, getrusage
 
         mem = getrusage(RUSAGE_SELF).ru_maxrss
         if sys.platform == "darwin":
@@ -209,7 +207,7 @@ class PillowLeakTestCase:
         for cycle in range(self.iterations):
             core()
             mem = self._get_mem_usage() - start_mem
-            msg = "memory usage limit exceeded in iteration %d" % cycle
+            msg = f"memory usage limit exceeded in iteration {cycle}"
             assert mem < self.mem_limit, msg
 
 
@@ -272,12 +270,8 @@ def on_github_actions():
 
 
 def on_ci():
-    # Travis and AppVeyor have "CI"
-    # Azure Pipelines has "TF_BUILD"
-    # GitHub Actions has "GITHUB_ACTIONS"
-    return (
-        "CI" in os.environ or "TF_BUILD" in os.environ or "GITHUB_ACTIONS" in os.environ
-    )
+    # GitHub Actions, Travis and AppVeyor have "CI"
+    return "CI" in os.environ
 
 
 def is_big_endian():
@@ -292,6 +286,10 @@ def is_pypy():
     return hasattr(sys, "pypy_translation_info")
 
 
+def is_mingw():
+    return sysconfig.get_platform() == "mingw"
+
+
 if sys.platform == "win32":
     IMCONVERT = os.environ.get("MAGICK_HOME", "")
     if IMCONVERT:
