@@ -25,14 +25,14 @@
 
 #include "Python.h"
 
-#include "Imaging.h"
+#include "libImaging/Imaging.h"
 
 /* -------------------------------------------------------------------- */
-/* Windows DIB support	*/
+/* Windows DIB support */
 
 #ifdef _WIN32
 
-#include "ImDib.h"
+#include "libImaging/ImDib.h"
 
 #if SIZEOF_VOID_P == 8
 #define F_HANDLE "K"
@@ -52,17 +52,19 @@ _new(const char* mode, int xsize, int ysize)
 {
     ImagingDisplayObject *display;
 
-    if (PyType_Ready(&ImagingDisplayType) < 0)
+    if (PyType_Ready(&ImagingDisplayType) < 0) {
         return NULL;
+    }
 
     display = PyObject_New(ImagingDisplayObject, &ImagingDisplayType);
-    if (display == NULL)
-	return NULL;
+    if (display == NULL) {
+        return NULL;
+    }
 
     display->dib = ImagingNewDIB(mode, xsize, ysize);
     if (!display->dib) {
-	Py_DECREF(display);
-	return NULL;
+        Py_DECREF(display);
+        return NULL;
     }
 
     return display;
@@ -71,8 +73,9 @@ _new(const char* mode, int xsize, int ysize)
 static void
 _delete(ImagingDisplayObject* display)
 {
-    if (display->dib)
-	ImagingDeleteDIB(display->dib);
+    if (display->dib) {
+        ImagingDeleteDIB(display->dib);
+    }
     PyObject_Del(display);
 }
 
@@ -80,8 +83,9 @@ static PyObject*
 _expose(ImagingDisplayObject* display, PyObject* args)
 {
     HDC hdc;
-    if (!PyArg_ParseTuple(args, F_HANDLE, &hdc))
-	return NULL;
+    if (!PyArg_ParseTuple(args, F_HANDLE, &hdc)) {
+        return NULL;
+    }
 
     ImagingExposeDIB(display->dib, hdc);
 
@@ -97,8 +101,9 @@ _draw(ImagingDisplayObject* display, PyObject* args)
     int src[4];
     if (!PyArg_ParseTuple(args, F_HANDLE "(iiii)(iiii)", &hdc,
                           dst+0, dst+1, dst+2, dst+3,
-                          src+0, src+1, src+2, src+3))
-	return NULL;
+                          src+0, src+1, src+2, src+3)) {
+        return NULL;
+    }
 
     ImagingDrawDIB(display->dib, hdc, dst, src);
 
@@ -116,16 +121,20 @@ _paste(ImagingDisplayObject* display, PyObject* args)
     PyObject* op;
     int xy[4];
     xy[0] = xy[1] = xy[2] = xy[3] = 0;
-    if (!PyArg_ParseTuple(args, "O|(iiii)", &op, xy+0, xy+1, xy+2, xy+3))
-	return NULL;
+    if (!PyArg_ParseTuple(args, "O|(iiii)", &op, xy+0, xy+1, xy+2, xy+3)) {
+        return NULL;
+    }
     im = PyImaging_AsImaging(op);
-    if (!im)
-	return NULL;
+    if (!im) {
+        return NULL;
+    }
 
-    if (xy[2] <= xy[0])
-	xy[2] = xy[0] + im->xsize;
-    if (xy[3] <= xy[1])
-	xy[3] = xy[1] + im->ysize;
+    if (xy[2] <= xy[0]) {
+        xy[2] = xy[0] + im->xsize;
+    }
+    if (xy[3] <= xy[1]) {
+        xy[3] = xy[1] + im->ysize;
+    }
 
     ImagingPasteDIB(display->dib, im, xy);
 
@@ -139,8 +148,9 @@ _query_palette(ImagingDisplayObject* display, PyObject* args)
     HDC hdc;
     int status;
 
-    if (!PyArg_ParseTuple(args, F_HANDLE, &hdc))
-	return NULL;
+    if (!PyArg_ParseTuple(args, F_HANDLE, &hdc)) {
+        return NULL;
+    }
 
     status = ImagingQueryPaletteDIB(display->dib, hdc);
 
@@ -153,12 +163,13 @@ _getdc(ImagingDisplayObject* display, PyObject* args)
     HWND window;
     HDC dc;
 
-    if (!PyArg_ParseTuple(args, F_HANDLE, &window))
-	return NULL;
+    if (!PyArg_ParseTuple(args, F_HANDLE, &window)) {
+        return NULL;
+    }
 
     dc = GetDC(window);
     if (!dc) {
-        PyErr_SetString(PyExc_IOError, "cannot create dc");
+        PyErr_SetString(PyExc_OSError, "cannot create dc");
         return NULL;
     }
 
@@ -171,8 +182,9 @@ _releasedc(ImagingDisplayObject* display, PyObject* args)
     HWND window;
     HDC dc;
 
-    if (!PyArg_ParseTuple(args, F_HANDLE F_HANDLE, &window, &dc))
-	return NULL;
+    if (!PyArg_ParseTuple(args, F_HANDLE F_HANDLE, &window, &dc)) {
+        return NULL;
+    }
 
     ReleaseDC(window, dc);
 
@@ -186,8 +198,9 @@ _frombytes(ImagingDisplayObject* display, PyObject* args)
     char* ptr;
     int bytes;
 
-    if (!PyArg_ParseTuple(args, "y#:frombytes", &ptr, &bytes))
+    if (!PyArg_ParseTuple(args, "y#:frombytes", &ptr, &bytes)) {
         return NULL;
+    }
 
     if (display->dib->ysize * display->dib->linesize != bytes) {
         PyErr_SetString(PyExc_ValueError, "wrong size");
@@ -203,8 +216,9 @@ _frombytes(ImagingDisplayObject* display, PyObject* args)
 static PyObject*
 _tobytes(ImagingDisplayObject* display, PyObject* args)
 {
-    if (!PyArg_ParseTuple(args, ":tobytes"))
+    if (!PyArg_ParseTuple(args, ":tobytes")) {
         return NULL;
+    }
 
     return PyBytes_FromStringAndSize(
         display->dib->bits, display->dib->ysize * display->dib->linesize
@@ -228,13 +242,13 @@ static struct PyMethodDef methods[] = {
 static PyObject*
 _getattr_mode(ImagingDisplayObject* self, void* closure)
 {
-	return Py_BuildValue("s", self->dib->mode);
+    return Py_BuildValue("s", self->dib->mode);
 }
 
 static PyObject*
 _getattr_size(ImagingDisplayObject* self, void* closure)
 {
-	return Py_BuildValue("ii", self->dib->xsize, self->dib->ysize);
+    return Py_BuildValue("ii", self->dib->xsize, self->dib->ysize);
 }
 
 static struct PyGetSetDef getsetters[] = {
@@ -244,13 +258,13 @@ static struct PyGetSetDef getsetters[] = {
 };
 
 static PyTypeObject ImagingDisplayType = {
-	PyVarObject_HEAD_INIT(NULL, 0)
-	"ImagingDisplay",		/*tp_name*/
-	sizeof(ImagingDisplayObject),	/*tp_size*/
-	0,				/*tp_itemsize*/
-	/* methods */
-	(destructor)_delete,		/*tp_dealloc*/
-	0,				/*tp_print*/
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "ImagingDisplay",            /*tp_name*/
+    sizeof(ImagingDisplayObject),/*tp_size*/
+    0,                           /*tp_itemsize*/
+    /* methods */
+    (destructor)_delete,        /*tp_dealloc*/
+    0,                          /*tp_print*/
     0,                          /*tp_getattr*/
     0,                          /*tp_setattr*/
     0,                          /*tp_compare*/
@@ -284,12 +298,14 @@ PyImaging_DisplayWin32(PyObject* self, PyObject* args)
     char *mode;
     int xsize, ysize;
 
-    if (!PyArg_ParseTuple(args, "s(ii)", &mode, &xsize, &ysize))
-	return NULL;
+    if (!PyArg_ParseTuple(args, "s(ii)", &mode, &xsize, &ysize)) {
+        return NULL;
+    }
 
     display = _new(mode, xsize, ysize);
-    if (display == NULL)
-	return NULL;
+    if (display == NULL) {
+        return NULL;
+    }
 
     return (PyObject*) display;
 }
@@ -324,8 +340,9 @@ PyImaging_GrabScreenWin32(PyObject* self, PyObject* args)
     HMODULE user32;
     Func_SetThreadDpiAwarenessContext SetThreadDpiAwarenessContext_function;
 
-    if (!PyArg_ParseTuple(args, "|ii", &includeLayeredWindows, &all_screens))
+    if (!PyArg_ParseTuple(args, "|ii", &includeLayeredWindows, &all_screens)) {
         return NULL;
+    }
 
     /* step 1: create a memory DC large enough to hold the
        entire screen */
@@ -361,25 +378,30 @@ PyImaging_GrabScreenWin32(PyObject* self, PyObject* args)
     FreeLibrary(user32);
 
     bitmap = CreateCompatibleBitmap(screen, width, height);
-    if (!bitmap)
+    if (!bitmap) {
         goto error;
+    }
 
-    if (!SelectObject(screen_copy, bitmap))
+    if (!SelectObject(screen_copy, bitmap)) {
         goto error;
+    }
 
     /* step 2: copy bits into memory DC bitmap */
 
     rop = SRCCOPY;
-    if (includeLayeredWindows)
+    if (includeLayeredWindows) {
         rop |= CAPTUREBLT;
-    if (!BitBlt(screen_copy, 0, 0, width, height, screen, x, y, rop))
+    }
+    if (!BitBlt(screen_copy, 0, 0, width, height, screen, x, y, rop)) {
         goto error;
+    }
 
     /* step 3: extract bits from bitmap */
 
     buffer = PyBytes_FromStringAndSize(NULL, height * ((width*3 + 3) & -4));
-    if (!buffer)
+    if (!buffer) {
         return NULL;
+    }
 
     core.bcSize = sizeof(core);
     core.bcWidth = width;
@@ -387,8 +409,9 @@ PyImaging_GrabScreenWin32(PyObject* self, PyObject* args)
     core.bcPlanes = 1;
     core.bcBitCount = 24;
     if (!GetDIBits(screen_copy, bitmap, 0, height, PyBytes_AS_STRING(buffer),
-                   (BITMAPINFO*) &core, DIB_RGB_COLORS))
+                   (BITMAPINFO*) &core, DIB_RGB_COLORS)) {
         goto error;
+    }
 
     DeleteObject(bitmap);
     DeleteDC(screen_copy);
@@ -397,7 +420,7 @@ PyImaging_GrabScreenWin32(PyObject* self, PyObject* args)
     return Py_BuildValue("(ii)(ii)N", x, y, width, height, buffer);
 
 error:
-    PyErr_SetString(PyExc_IOError, "screen grab failed");
+    PyErr_SetString(PyExc_OSError, "screen grab failed");
 
     DeleteDC(screen_copy);
     DeleteDC(screen);
@@ -418,12 +441,15 @@ static BOOL CALLBACK list_windows_callback(HWND hwnd, LPARAM lParam)
     title_size = GetWindowTextLength(hwnd);
     if (title_size > 0) {
         title = PyUnicode_FromStringAndSize(NULL, title_size);
-        if (title)
+        if (title) {
             GetWindowTextW(hwnd, PyUnicode_AS_UNICODE(title), title_size+1);
-    } else
+        }
+    } else {
         title = PyUnicode_FromString("");
-    if (!title)
+    }
+    if (!title) {
         return 0;
+    }
 
     /* get bounding boxes */
     GetClientRect(hwnd, &inner);
@@ -434,15 +460,17 @@ static BOOL CALLBACK list_windows_callback(HWND hwnd, LPARAM lParam)
         inner.left, inner.top, inner.right, inner.bottom,
         outer.left, outer.top, outer.right, outer.bottom
         );
-    if (!item)
+    if (!item) {
         return 0;
+    }
 
     status = PyList_Append(window_list, item);
 
     Py_DECREF(item);
 
-    if (status < 0)
+    if (status < 0) {
         return 0;
+    }
 
     return 1;
 }
@@ -453,8 +481,9 @@ PyImaging_ListWindowsWin32(PyObject* self, PyObject* args)
     PyObject* window_list;
 
     window_list = PyList_New(0);
-    if (!window_list)
+    if (!window_list) {
         return NULL;
+    }
 
     EnumWindows(list_windows_callback, (LPARAM) window_list);
 
@@ -473,33 +502,45 @@ PyObject*
 PyImaging_GrabClipboardWin32(PyObject* self, PyObject* args)
 {
     int clip;
-    HANDLE handle;
+    HANDLE handle = NULL;
     int size;
     void* data;
     PyObject* result;
+    UINT format;
+    UINT formats[] = { CF_DIB, CF_DIBV5, CF_HDROP, RegisterClipboardFormatA("PNG"), 0 };
+    LPCSTR format_names[] = { "DIB", "DIB", "file", "png", NULL };
 
-    clip = OpenClipboard(NULL);
-    /* FIXME: check error status */
+    if (!OpenClipboard(NULL)) {
+        PyErr_SetString(PyExc_OSError, "failed to open clipboard");
+        return NULL;
+    }
 
-    handle = GetClipboardData(CF_DIB);
+    // find best format as set by clipboard owner
+    format = 0;
+    while (!handle && (format = EnumClipboardFormats(format))) {
+        for (UINT i = 0; formats[i] != 0; i++) {
+            if (format == formats[i]) {
+                handle = GetClipboardData(format);
+                format = i;
+                break;
+            }
+        }
+    }
+
     if (!handle) {
-        /* FIXME: add CF_HDROP support to allow cut-and-paste from
-           the explorer */
         CloseClipboard();
-        Py_INCREF(Py_None);
-        return Py_None;
+        return Py_BuildValue("zO", NULL, Py_None);
     }
 
-    size = GlobalSize(handle);
     data = GlobalLock(handle);
+    size = GlobalSize(handle);
 
     result = PyBytes_FromStringAndSize(data, size);
 
     GlobalUnlock(handle);
-
     CloseClipboard();
 
-    return result;
+    return Py_BuildValue("zN", format_names[format], result);
 }
 
 /* -------------------------------------------------------------------- */
@@ -556,8 +597,9 @@ windowCallback(HWND wnd, UINT message, WPARAM wParam, LPARAM lParam)
                 GetWindowLongPtr(wnd, sizeof(PyObject*));
             current_threadstate = PyThreadState_Swap(NULL);
             PyEval_RestoreThread(threadstate);
-        } else
+        } else {
             return DefWindowProc(wnd, message, wParam, lParam);
+        }
     }
 
     /* process message */
@@ -575,28 +617,31 @@ windowCallback(HWND wnd, UINT message, WPARAM wParam, LPARAM lParam)
             ps.rcPaint.left, ps.rcPaint.top,
             ps.rcPaint.right, ps.rcPaint.bottom
             );
-        if (result)
+        if (result) {
             Py_DECREF(result);
-        else
+        } else {
             callback_error("window damage callback");
+        }
 
         result = PyObject_CallFunction(
             callback, "s" F_HANDLE "iiii", "clear", dc,
             0, 0, rect.right-rect.left, rect.bottom-rect.top
             );
-        if (result)
+        if (result) {
             Py_DECREF(result);
-        else
+        } else {
             callback_error("window clear callback");
+        }
 
         result = PyObject_CallFunction(
             callback, "s" F_HANDLE "iiii", "repair", dc,
             0, 0, rect.right-rect.left, rect.bottom-rect.top
             );
-        if (result)
+        if (result) {
             Py_DECREF(result);
-        else
+        } else {
             callback_error("window repair callback");
+        }
 
         ReleaseDC(wnd, dc);
         EndPaint(wnd, &ps);
@@ -610,17 +655,19 @@ windowCallback(HWND wnd, UINT message, WPARAM wParam, LPARAM lParam)
         if (result) {
             InvalidateRect(wnd, NULL, 1);
             Py_DECREF(result);
-        } else
+        } else {
             callback_error("window resize callback");
+        }
         break;
 
     case WM_DESTROY:
         /* destroy window */
         result = PyObject_CallFunction(callback, "s", "destroy");
-        if (result)
+        if (result) {
             Py_DECREF(result);
-        else
+        } else {
             callback_error("window destroy callback");
+        }
         Py_DECREF(callback);
         break;
 
@@ -646,13 +693,16 @@ PyImaging_CreateWindowWin32(PyObject* self, PyObject* args)
     char* title;
     PyObject* callback;
     int width = 0, height = 0;
-    if (!PyArg_ParseTuple(args, "sO|ii", &title, &callback, &width, &height))
-	return NULL;
+    if (!PyArg_ParseTuple(args, "sO|ii", &title, &callback, &width, &height)) {
+        return NULL;
+    }
 
-    if (width <= 0)
+    if (width <= 0) {
         width = CW_USEDEFAULT;
-    if (height <= 0)
+    }
+    if (height <= 0) {
         height = CW_USEDEFAULT;
+    }
 
     /* register toplevel window class */
     windowClass.style = CS_CLASSDC;
@@ -677,7 +727,7 @@ PyImaging_CreateWindowWin32(PyObject* self, PyObject* args)
         );
 
     if (!wnd) {
-        PyErr_SetString(PyExc_IOError, "failed to create window");
+        PyErr_SetString(PyExc_OSError, "failed to create window");
         return NULL;
     }
 
@@ -731,8 +781,9 @@ PyImaging_DrawWmf(PyObject* self, PyObject* args)
     int width, height;
     int x0, y0, x1, y1;
     if (!PyArg_ParseTuple(args, "y#(ii)(iiii):_load", &data, &datasize,
-                          &width, &height, &x0, &x1, &y0, &y1))
+                          &width, &height, &x0, &x1, &y0, &y1)) {
         return NULL;
+    }
 
     /* step 1: copy metafile contents into METAFILE object */
 
@@ -755,7 +806,7 @@ PyImaging_DrawWmf(PyObject* self, PyObject* args)
     }
 
     if (!meta) {
-        PyErr_SetString(PyExc_IOError, "cannot load metafile");
+        PyErr_SetString(PyExc_OSError, "cannot load metafile");
         return NULL;
     }
 
@@ -774,12 +825,12 @@ PyImaging_DrawWmf(PyObject* self, PyObject* args)
         );
 
     if (!bitmap) {
-        PyErr_SetString(PyExc_IOError, "cannot create bitmap");
+        PyErr_SetString(PyExc_OSError, "cannot create bitmap");
         goto error;
     }
 
     if (!SelectObject(dc, bitmap)) {
-        PyErr_SetString(PyExc_IOError, "cannot select bitmap");
+        PyErr_SetString(PyExc_OSError, "cannot select bitmap");
         goto error;
     }
 
@@ -793,7 +844,7 @@ PyImaging_DrawWmf(PyObject* self, PyObject* args)
     FillRect(dc, &rect, GetStockObject(WHITE_BRUSH));
 
     if (!PlayEnhMetaFile(dc, meta, &rect)) {
-        PyErr_SetString(PyExc_IOError, "cannot render metafile");
+        PyErr_SetString(PyExc_OSError, "cannot render metafile");
         goto error;
     }
 
@@ -806,8 +857,9 @@ PyImaging_DrawWmf(PyObject* self, PyObject* args)
 error:
     DeleteEnhMetaFile(meta);
 
-    if (bitmap)
+    if (bitmap) {
         DeleteObject(bitmap);
+    }
 
     DeleteDC(dc);
 
@@ -817,7 +869,7 @@ error:
 #endif /* _WIN32 */
 
 /* -------------------------------------------------------------------- */
-/* X11 support	*/
+/* X11 support */
 
 #ifdef HAVE_XCB
 #include <xcb/xcb.h>
@@ -838,14 +890,15 @@ PyImaging_GrabScreenX11(PyObject* self, PyObject* args)
     xcb_generic_error_t* error;
     PyObject* buffer = NULL;
 
-    if (!PyArg_ParseTuple(args, "|z", &display_name))
+    if (!PyArg_ParseTuple(args, "|z", &display_name)) {
         return NULL;
+    }
 
     /* connect to X and get screen data */
 
     connection = xcb_connect(display_name, &screen_number);
     if (xcb_connection_has_error(connection)) {
-        PyErr_Format(PyExc_IOError, "X connection failed: error %i", xcb_connection_has_error(connection));
+        PyErr_Format(PyExc_OSError, "X connection failed: error %i", xcb_connection_has_error(connection));
         xcb_disconnect(connection);
         return NULL;
     }
@@ -860,7 +913,7 @@ PyImaging_GrabScreenX11(PyObject* self, PyObject* args)
     if (screen == NULL || screen->root == 0) {
         // this case is usually caught with "X connection failed: error 6" above
         xcb_disconnect(connection);
-        PyErr_SetString(PyExc_IOError, "X screen not found");
+        PyErr_SetString(PyExc_OSError, "X screen not found");
         return NULL;
     }
 
@@ -874,7 +927,7 @@ PyImaging_GrabScreenX11(PyObject* self, PyObject* args)
                                               0, 0, width, height, 0x00ffffff),
                                 &error);
     if (reply == NULL) {
-        PyErr_Format(PyExc_IOError, "X get_image failed: error %i (%i, %i, %i)",
+        PyErr_Format(PyExc_OSError, "X get_image failed: error %i (%i, %i, %i)",
                      error->error_code, error->major_code, error->minor_code, error->resource_id);
         free(error);
         xcb_disconnect(connection);
@@ -887,14 +940,15 @@ PyImaging_GrabScreenX11(PyObject* self, PyObject* args)
         buffer = PyBytes_FromStringAndSize((char*)xcb_get_image_data(reply),
                                            xcb_get_image_data_length(reply));
     } else {
-        PyErr_Format(PyExc_IOError, "unsupported bit depth: %i", reply->depth);
+        PyErr_Format(PyExc_OSError, "unsupported bit depth: %i", reply->depth);
     }
 
     free(reply);
     xcb_disconnect(connection);
 
-    if (!buffer)
+    if (!buffer) {
         return NULL;
+    }
 
     return Py_BuildValue("(ii)N", width, height, buffer);
 }
