@@ -35,6 +35,7 @@
 #include "Imaging.h"
 
 #include <math.h>
+#include <stdint.h>
 
 #define CEIL(v)  (int) ceil(v)
 #define FLOOR(v) ((v) >= 0.0 ? (int) (v) : (int) floor(v))
@@ -79,8 +80,9 @@ point8(Imaging im, int x, int y, int ink)
 static inline void
 point32(Imaging im, int x, int y, int ink)
 {
-    if (x >= 0 && x < im->xsize && y >= 0 && y < im->ysize)
+    if (x >= 0 && x < im->xsize && y >= 0 && y < im->ysize) {
         im->image32[y][x] = ink;
+    }
 }
 
 static inline void
@@ -103,16 +105,19 @@ hline8(Imaging im, int x0, int y0, int x1, int ink)
     int tmp, pixelwidth;
 
     if (y0 >= 0 && y0 < im->ysize) {
-        if (x0 > x1)
+        if (x0 > x1) {
             tmp = x0, x0 = x1, x1 = tmp;
-        if (x0 < 0)
+        }
+        if (x0 < 0) {
             x0 = 0;
-        else if (x0 >= im->xsize)
+        } else if (x0 >= im->xsize) {
             return;
-        if (x1 < 0)
+        }
+        if (x1 < 0) {
             return;
-        else if (x1 >= im->xsize)
+        } else if (x1 >= im->xsize) {
             x1 = im->xsize-1;
+        }
         if (x0 <= x1) {
             pixelwidth = strncmp(im->mode, "I;16", 4) == 0 ? 2 : 1;
             memset(im->image8[y0] + x0 * pixelwidth, (UINT8) ink,
@@ -128,19 +133,23 @@ hline32(Imaging im, int x0, int y0, int x1, int ink)
     INT32* p;
 
     if (y0 >= 0 && y0 < im->ysize) {
-        if (x0 > x1)
+        if (x0 > x1) {
             tmp = x0, x0 = x1, x1 = tmp;
-        if (x0 < 0)
+        }
+        if (x0 < 0) {
             x0 = 0;
-        else if (x0 >= im->xsize)
+        } else if (x0 >= im->xsize) {
             return;
-        if (x1 < 0)
+        }
+        if (x1 < 0) {
             return;
-        else if (x1 >= im->xsize)
+        } else if (x1 >= im->xsize) {
             x1 = im->xsize-1;
+        }
         p = im->image32[y0];
-        while (x0 <= x1)
+        while (x0 <= x1) {
             p[x0++] = ink;
+        }
     }
 }
 
@@ -151,16 +160,19 @@ hline32rgba(Imaging im, int x0, int y0, int x1, int ink)
     unsigned int tmp1;
 
     if (y0 >= 0 && y0 < im->ysize) {
-        if (x0 > x1)
+        if (x0 > x1) {
             tmp = x0, x0 = x1, x1 = tmp;
-        if (x0 < 0)
+        }
+        if (x0 < 0) {
             x0 = 0;
-        else if (x0 >= im->xsize)
+        } else if (x0 >= im->xsize) {
             return;
-        if (x1 < 0)
+        }
+        if (x1 < 0) {
             return;
-        else if (x1 >= im->xsize)
+        } else if (x1 >= im->xsize) {
             x1 = im->xsize-1;
+        }
         if (x0 <= x1) {
             UINT8* out = (UINT8*) im->image[y0]+x0*4;
             UINT8* in = (UINT8*) &ink;
@@ -183,19 +195,21 @@ line8(Imaging im, int x0, int y0, int x1, int y1, int ink)
 
     /* normalize coordinates */
     dx = x1-x0;
-    if (dx < 0)
+    if (dx < 0) {
         dx = -dx, xs = -1;
-    else
+    } else {
         xs = 1;
+    }
     dy = y1-y0;
-    if (dy < 0)
+    if (dy < 0) {
         dy = -dy, ys = -1;
-    else
+    } else {
         ys = 1;
+    }
 
     n = (dx > dy) ? dx : dy;
 
-    if (dx == 0)
+    if (dx == 0) {
 
         /* vertical */
         for (i = 0; i < dy; i++) {
@@ -203,7 +217,7 @@ line8(Imaging im, int x0, int y0, int x1, int y1, int ink)
             y0 += ys;
         }
 
-    else if (dy == 0)
+    } else if (dy == 0) {
 
         /* horizontal */
         for (i = 0; i < dx; i++) {
@@ -211,7 +225,7 @@ line8(Imaging im, int x0, int y0, int x1, int y1, int ink)
             x0 += xs;
         }
 
-    else if (dx > dy) {
+    } else if (dx > dy) {
 
         /* bresenham, horizontal slope */
         n = dx;
@@ -259,19 +273,21 @@ line32(Imaging im, int x0, int y0, int x1, int y1, int ink)
 
     /* normalize coordinates */
     dx = x1-x0;
-    if (dx < 0)
+    if (dx < 0) {
         dx = -dx, xs = -1;
-    else
+    } else {
         xs = 1;
+    }
     dy = y1-y0;
-    if (dy < 0)
+    if (dy < 0) {
         dy = -dy, ys = -1;
-    else
+    } else {
         ys = 1;
+    }
 
     n = (dx > dy) ? dx : dy;
 
-    if (dx == 0)
+    if (dx == 0) {
 
         /* vertical */
         for (i = 0; i < dy; i++) {
@@ -279,7 +295,7 @@ line32(Imaging im, int x0, int y0, int x1, int y1, int ink)
             y0 += ys;
         }
 
-    else if (dy == 0)
+    } else if (dy == 0) {
 
         /* horizontal */
         for (i = 0; i < dx; i++) {
@@ -287,7 +303,7 @@ line32(Imaging im, int x0, int y0, int x1, int y1, int ink)
             x0 += xs;
         }
 
-    else if (dx > dy) {
+    } else if (dx > dy) {
 
         /* bresenham, horizontal slope */
         n = dx;
@@ -335,19 +351,21 @@ line32rgba(Imaging im, int x0, int y0, int x1, int y1, int ink)
 
     /* normalize coordinates */
     dx = x1-x0;
-    if (dx < 0)
+    if (dx < 0) {
         dx = -dx, xs = -1;
-    else
+    } else {
         xs = 1;
+    }
     dy = y1-y0;
-    if (dy < 0)
+    if (dy < 0) {
         dy = -dy, ys = -1;
-    else
+    } else {
         ys = 1;
+    }
 
     n = (dx > dy) ? dx : dy;
 
-    if (dx == 0)
+    if (dx == 0) {
 
         /* vertical */
         for (i = 0; i < dy; i++) {
@@ -355,7 +373,7 @@ line32rgba(Imaging im, int x0, int y0, int x1, int y1, int ink)
             y0 += ys;
         }
 
-    else if (dy == 0)
+    } else if (dy == 0) {
 
         /* horizontal */
         for (i = 0; i < dx; i++) {
@@ -363,7 +381,7 @@ line32rgba(Imaging im, int x0, int y0, int x1, int y1, int ink)
             x0 += xs;
         }
 
-    else if (dx > dy) {
+    } else if (dx > dy) {
 
         /* bresenham, horizontal slope */
         n = dx;
@@ -406,12 +424,13 @@ static int
 x_cmp(const void *x0, const void *x1)
 {
     float diff = *((float*)x0) - *((float*)x1);
-    if (diff < 0)
+    if (diff < 0) {
         return -1;
-    else if (diff > 0)
+    } else if (diff > 0) {
         return 1;
-    else
+    } else {
         return 0;
+    }
 }
 
 
@@ -471,15 +490,15 @@ polygon_generic(Imaging im, int n, Edge *e, int ink, int eofill,
     }
 
     for (i = 0; i < n; i++) {
-        if (e[i].ymin == e[i].ymax) {
-            continue;
-        }
         if (ymin > e[i].ymin) {
             ymin = e[i].ymin;
         }
         if (ymax < e[i].ymax) {
             ymax = e[i].ymax;
         }
+        if (e[i].ymin == e[i].ymax) {
+            continue;
+        }
         edge_table[edge_count++] = (e + i);
     }
     if (ymin < 0) {
@@ -566,25 +585,28 @@ add_edge(Edge *e, int x0, int y0, int x1, int y1)
 {
     /* printf("edge %d %d %d %d\n", x0, y0, x1, y1); */
 
-    if (x0 <= x1)
+    if (x0 <= x1) {
         e->xmin = x0, e->xmax = x1;
-    else
+    } else {
         e->xmin = x1, e->xmax = x0;
+    }
 
-    if (y0 <= y1)
+    if (y0 <= y1) {
         e->ymin = y0, e->ymax = y1;
-    else
+    } else {
         e->ymin = y1, e->ymax = y0;
+    }
 
     if (y0 == y1) {
         e->d = 0;
         e->dx = 0.0;
     } else {
         e->dx = ((float)(x1-x0)) / (y1-y0);
-        if (y0 == e->ymin)
+        if (y0 == e->ymin) {
             e->d = 1;
-        else
+        } else {
             e->d = -1;
+        }
     }
 
     e->x0 = x0;
@@ -662,7 +684,7 @@ ImagingDrawWideLine(Imaging im, int x0, int y0, int x1, int y1,
         return 0;
     }
 
-    big_hypotenuse = sqrt((double) (dx*dx + dy*dy));
+    big_hypotenuse = hypot(dx, dy);
     small_hypotenuse = (width - 1) / 2.0;
     ratio_max = ROUND_UP(small_hypotenuse) / big_hypotenuse;
     ratio_min = ROUND_DOWN(small_hypotenuse) / big_hypotenuse;
@@ -701,23 +723,27 @@ ImagingDrawRectangle(Imaging im, int x0, int y0, int x1, int y1,
 
     DRAWINIT();
 
-    if (y0 > y1)
+    if (y0 > y1) {
         tmp = y0, y0 = y1, y1 = tmp;
+    }
 
     if (fill) {
 
-        if (y0 < 0)
+        if (y0 < 0) {
             y0 = 0;
-        else if (y0 >= im->ysize)
+        } else if (y0 >= im->ysize) {
             return 0;
+        }
 
-        if (y1 < 0)
+        if (y1 < 0) {
             return 0;
-        else if (y1 > im->ysize)
+        } else if (y1 > im->ysize) {
             y1 = im->ysize;
+        }
 
-        for (y = y0; y <= y1; y++)
+        for (y = y0; y <= y1; y++) {
             draw->hline(im, x0, y, x1, ink);
+        }
 
     } else {
         /* outline */
@@ -743,8 +769,9 @@ ImagingDrawPolygon(Imaging im, int count, int* xy, const void* ink_,
     DRAW* draw;
     INT32 ink;
 
-    if (count <= 0)
+    if (count <= 0) {
         return 0;
+    }
 
     DRAWINIT();
 
@@ -757,18 +784,21 @@ ImagingDrawPolygon(Imaging im, int count, int* xy, const void* ink_,
             (void) ImagingError_MemoryError();
             return -1;
         }
-        for (i = n = 0; i < count-1; i++)
+        for (i = n = 0; i < count-1; i++) {
             add_edge(&e[n++], xy[i+i], xy[i+i+1], xy[i+i+2], xy[i+i+3]);
-        if (xy[i+i] != xy[0] || xy[i+i+1] != xy[1])
+        }
+        if (xy[i+i] != xy[0] || xy[i+i+1] != xy[1]) {
             add_edge(&e[n++], xy[i+i], xy[i+i+1], xy[0], xy[1]);
+        }
         draw->polygon(im, n, e, ink, 0);
         free(e);
 
     } else {
 
         /* Outline */
-        for (i = 0; i < count-1; i++)
+        for (i = 0; i < count-1; i++) {
             draw->line(im, xy[i+i], xy[i+i+1], xy[i+i+2], xy[i+i+3], ink);
+        }
         draw->line(im, xy[i+i], xy[i+i+1], xy[0], xy[1], ink);
 
     }
@@ -789,216 +819,751 @@ ImagingDrawBitmap(Imaging im, int x0, int y0, Imaging bitmap, const void* ink,
 /* -------------------------------------------------------------------- */
 /* standard shapes */
 
-#define ARC 0
-#define CHORD 1
-#define PIESLICE 2
+// Imagine 2D plane and ellipse with center in (0, 0) and semi-major axes a and b.
+// Then quarter_* stuff approximates its top right quarter (x, y >= 0) with integer
+// points from set {(2x+x0, 2y+y0) | x,y in Z} where x0, y0 are from {0, 1} and
+// are such that point (a, b) is in the set.
 
-static void
-ellipsePoint(int cx, int cy, int w, int h,
-             float i, int *x, int *y)
-{
-    float i_cos, i_sin;
-    float x_f, y_f;
-    double modf_int;
-    i_cos = cos(i*M_PI/180);
-    i_sin = sin(i*M_PI/180);
-    x_f = (i_cos * w/2) + cx;
-    y_f = (i_sin * h/2) + cy;
-    if (modf(x_f, &modf_int) == 0.5) {
-        *x = i_cos > 0 ? FLOOR(x_f) : CEIL(x_f);
+typedef struct {
+    int32_t a, b, cx, cy, ex, ey;
+    int64_t a2, b2, a2b2;
+    int8_t finished;
+} quarter_state;
+
+void quarter_init(quarter_state* s, int32_t a, int32_t b) {
+    if (a < 0 || b < 0) {
+        s->finished = 1;
     } else {
-        *x = FLOOR(x_f + 0.5);
-    }
-    if (modf(y_f, &modf_int) == 0.5) {
-        *y = i_sin > 0 ? FLOOR(y_f) : CEIL(y_f);
-    } else {
-        *y = FLOOR(y_f + 0.5);
+        s->a = a;
+        s->b = b;
+        s->cx = a;
+        s->cy = b % 2;
+        s->ex = a % 2;
+        s->ey = b;
+        s->a2 = a * a;
+        s->b2 = b * b;
+        s->a2b2 = s->a2 * s->b2;
+        s->finished = 0;
     }
 }
 
-static int
-ellipse(Imaging im, int x0, int y0, int x1, int y1,
-        float start, float end, const void* ink_, int fill,
-        int width, int mode, int op)
-{
-    float i;
-    int inner;
-    int n;
-    int maxEdgeCount;
-    int w, h;
-    int x, y;
-    int cx, cy;
-    int lx = 0, ly = 0;
-    int sx = 0, sy = 0;
-    int lx_inner = 0, ly_inner = 0;
-    int sx_inner = 0, sy_inner = 0;
-    DRAW* draw;
-    INT32 ink;
-    Edge* e;
+// deviation of the point from ellipse curve, basically a substitution
+// of the point into the ellipse equation
+int64_t quarter_delta(quarter_state* s, int64_t x, int64_t y) {
+    return llabs(s->a2 * y * y + s->b2 * x * x - s->a2b2);
+}
 
-    DRAWINIT();
-
-    while (end < start)
-        end += 360;
-
-    if (end - start > 360) {
-        // no need to go in loops
-        end = start + 361;
+int8_t quarter_next(quarter_state* s, int32_t* ret_x, int32_t* ret_y) {
+    if (s->finished) {
+        return -1;
     }
-
-    w = x1 - x0;
-    h = y1 - y0;
-    if (w <= 0 || h <= 0)
-        return 0;
-
-    cx = (x0 + x1) / 2;
-    cy = (y0 + y1) / 2;
-
-    if (!fill && width <= 1) {
-        for (i = start; i < end+1; i++) {
-            if (i > end) {
-                i = end;
-            }
-            ellipsePoint(cx, cy, w, h, i, &x, &y);
-            if (i != start)
-                draw->line(im, lx, ly, x, y, ink);
-            else
-                sx = x, sy = y;
-            lx = x, ly = y;
-        }
-
-        if (i != start) {
-            if (mode == PIESLICE) {
-                if (x != cx || y != cy) {
-                    draw->line(im, x, y, cx, cy, ink);
-                    draw->line(im, cx, cy, sx, sy, ink);
-                }
-            } else if (mode == CHORD) {
-                if (x != sx || y != sy)
-                    draw->line(im, x, y, sx, sy, ink);
-            }
-        }
+    *ret_x = s->cx;
+    *ret_y = s->cy;
+    if (s->cx == s->ex && s->cy == s->ey) {
+        s->finished = 1;
     } else {
-        inner = (mode == ARC || !fill) ? 1 : 0;
-
-        // Build edge list
-        // malloc check UNDONE, FLOAT?
-        maxEdgeCount = ceil(end - start);
-        if (inner) {
-            maxEdgeCount *= 2;
-        }
-        maxEdgeCount += 3;
-        e = calloc(maxEdgeCount, sizeof(Edge));
-        if (!e) {
-            ImagingError_MemoryError();
-            return -1;
-        }
-
-        // Outer circle
-        n = 0;
-        for (i = start; i < end+1; i++) {
-            if (i > end) {
-                i = end;
+        // Bresenham's algorithm, possible optimization: only consider 2 of 3
+        // next points depending on current slope
+        int32_t nx = s->cx;
+        int32_t ny = s->cy + 2;
+        int64_t ndelta = quarter_delta(s, nx, ny);
+        if (nx > 1) {
+            int64_t newdelta = quarter_delta(s, s->cx - 2, s->cy + 2);
+            if (ndelta > newdelta) {
+                nx = s->cx - 2;
+                ny = s->cy + 2;
+                ndelta = newdelta;
             }
-            ellipsePoint(cx, cy, w, h, i, &x, &y);
-            if (i == start) {
-                sx = x, sy = y;
-            } else {
-                add_edge(&e[n++], lx, ly, x, y);
-            }
-            lx = x, ly = y;
-        }
-        if (n == 0)
-            return 0;
-
-        if (inner) {
-            // Inner circle
-            x0 += width - 1;
-            y0 += width - 1;
-            x1 -= width - 1;
-            y1 -= width - 1;
-
-            w = x1 - x0;
-            h = y1 - y0;
-            if (w <= 0 || h <= 0) {
-                // ARC with no gap in the middle is a PIESLICE
-                mode = PIESLICE;
-                inner = 0;
-            } else {
-                for (i = start; i < end+1; i++) {
-                    if (i > end) {
-                        i = end;
-                    }
-                    ellipsePoint(cx, cy, w, h, i, &x, &y);
-                    if (i == start)
-                        sx_inner = x, sy_inner = y;
-                    else
-                        add_edge(&e[n++], lx_inner, ly_inner, x, y);
-                    lx_inner = x, ly_inner = y;
-                }
+            newdelta = quarter_delta(s, s->cx - 2, s->cy);
+            if (ndelta > newdelta) {
+                nx = s->cx - 2;
+                ny = s->cy;
             }
         }
-
-        if (end - start < 360) {
-            // Close polygon
-            if (mode == PIESLICE) {
-                if (x != cx || y != cy) {
-                    add_edge(&e[n++], sx, sy, cx, cy);
-                    add_edge(&e[n++], cx, cy, lx, ly);
-                    if (inner) {
-                        ImagingDrawWideLine(im, sx, sy, cx, cy, &ink, width, op);
-                        ImagingDrawWideLine(im, cx, cy, lx, ly, &ink, width, op);
-                    }
-                }
-            } else if (mode == CHORD) {
-                add_edge(&e[n++], sx, sy, lx, ly);
-                if (inner) {
-                    add_edge(&e[n++], sx_inner, sy_inner, lx_inner, ly_inner);
-                }
-            } else if (mode == ARC) {
-                add_edge(&e[n++], sx, sy, sx_inner, sy_inner);
-                add_edge(&e[n++], lx, ly, lx_inner, ly_inner);
-            }
-        }
-
-        draw->polygon(im, n, e, ink, 0);
-
-        free(e);
+        s->cx = nx;
+        s->cy = ny;
     }
-
     return 0;
 }
 
-int
-ImagingDrawArc(Imaging im, int x0, int y0, int x1, int y1,
-               float start, float end, const void* ink, int width, int op)
-{
-    return ellipse(im, x0, y0, x1, y1, start, end, ink, 0, width, ARC, op);
+// quarter_* stuff can "draw" a quarter of an ellipse with thickness 1, great.
+// Now we use ellipse_* stuff to join all four quarters of two different sized
+// ellipses and receive horizontal segments of a complete ellipse with
+// specified thickness.
+//
+// Still using integer grid with step 2 at this point (like in quarter_*)
+// to ease angle clipping in future.
+
+typedef struct {
+    quarter_state st_o, st_i;
+    int32_t py, pl, pr;
+    int32_t cy[4], cl[4], cr[4];
+    int8_t bufcnt;
+    int8_t finished;
+    int8_t leftmost;
+} ellipse_state;
+
+void ellipse_init(ellipse_state* s, int32_t a, int32_t b, int32_t w) {
+    s->bufcnt = 0;
+    s->leftmost = a % 2;
+    quarter_init(&s->st_o, a, b);
+    if (w < 1 || quarter_next(&s->st_o, &s->pr, &s->py) == -1) {
+        s->finished = 1;
+    } else {
+        s->finished = 0;
+        quarter_init(&s->st_i, a - 2 * (w - 1), b - 2 * (w - 1));
+        s->pl = s->leftmost;
+    }
 }
 
-int
-ImagingDrawChord(Imaging im, int x0, int y0, int x1, int y1,
-                 float start, float end, const void* ink, int fill,
-                 int width, int op)
+int8_t ellipse_next(ellipse_state* s, int32_t* ret_x0, int32_t* ret_y, int32_t* ret_x1) {
+    if (s->bufcnt == 0) {
+        if (s->finished) {
+            return -1;
+        }
+        int32_t y = s->py;
+        int32_t l = s->pl;
+        int32_t r = s->pr;
+        int32_t cx = 0, cy = 0;
+        int8_t next_ret;
+        while ((next_ret = quarter_next(&s->st_o, &cx, &cy)) != -1 && cy <= y) {
+        }
+        if (next_ret == -1) {
+            s->finished = 1;
+        } else {
+            s->pr = cx;
+            s->py = cy;
+        }
+        while ((next_ret = quarter_next(&s->st_i, &cx, &cy)) != -1 && cy <= y) {
+            l = cx;
+        }
+        s->pl = next_ret == -1 ? s->leftmost : cx;
+
+        if ((l > 0 || l < r) && y > 0) {
+            s->cl[s->bufcnt] = l == 0 ? 2 : l;
+            s->cy[s->bufcnt] = y;
+            s->cr[s->bufcnt] = r;
+            ++s->bufcnt;
+        }
+        if (y > 0) {
+            s->cl[s->bufcnt] = -r;
+            s->cy[s->bufcnt] = y;
+            s->cr[s->bufcnt] = -l;
+            ++s->bufcnt;
+        }
+        if (l > 0 || l < r) {
+            s->cl[s->bufcnt] = l == 0 ? 2 : l;
+            s->cy[s->bufcnt] = -y;
+            s->cr[s->bufcnt] = r;
+            ++s->bufcnt;
+        }
+        s->cl[s->bufcnt] = -r;
+        s->cy[s->bufcnt] = -y;
+        s->cr[s->bufcnt] = -l;
+        ++s->bufcnt;
+    }
+    --s->bufcnt;
+    *ret_x0 = s->cl[s->bufcnt];
+    *ret_y = s->cy[s->bufcnt];
+    *ret_x1 = s->cr[s->bufcnt];
+    return 0;
+}
+
+// Clipping tree consists of half-plane clipping nodes and combining nodes.
+// We can throw a horizontal segment in such a tree and collect an ordered set
+// of resulting disjoint clipped segments organized into a sorted linked list
+// of their end points.
+typedef enum {
+  CT_AND, // intersection
+  CT_OR, // union
+  CT_CLIP // half-plane clipping
+} clip_type;
+
+typedef struct clip_node {
+  clip_type type;
+  double a, b, c; // half-plane coeffs, only used in clipping nodes
+  struct clip_node* l; // child pointers, are only non-NULL in combining nodes
+  struct clip_node* r;
+} clip_node;
+
+// Linked list for the ends of the clipped horizontal segments.
+// Since the segment is always horizontal, we don't need to store Y coordinate.
+typedef struct event_list {
+  int32_t x;
+  int8_t type; // used internally, 1 for the left end (smaller X), -1 for the
+               // right end; pointless in output since the output segments
+               // are disjoint, therefore the types would always come in pairs
+               // and interchange (1 -1 1 -1 ...)
+  struct event_list* next;
+} event_list;
+
+// Mirrors all the clipping nodes of the tree relative to the y = x line.
+void clip_tree_transpose(clip_node* root) {
+  if (root != NULL) {
+    if (root->type == CT_CLIP) {
+      double t = root->a;
+      root->a = root->b;
+      root->b = t;
+    }
+    clip_tree_transpose(root->l);
+    clip_tree_transpose(root->r);
+  }
+}
+
+// Outputs a sequence of open-close events (types -1 and 1) for
+// non-intersecting segments sorted by X coordinate.
+// Combining nodes (AND, OR) may also accept sequences for intersecting
+// segments, i.e. something like correct bracket sequences.
+int clip_tree_do_clip(clip_node* root, int32_t x0, int32_t y, int32_t x1, event_list** ret) {
+  if (root == NULL) {
+    event_list* start = malloc(sizeof(event_list));
+    if (!start) {
+      ImagingError_MemoryError();
+      return -1;
+    }
+    event_list* end = malloc(sizeof(event_list));
+    if (!end) {
+      free(start);
+      ImagingError_MemoryError();
+      return -1;
+    }
+    start->x = x0;
+    start->type = 1;
+    start->next = end;
+    end->x = x1;
+    end->type = -1;
+    end->next = NULL;
+    *ret = start;
+    return 0;
+  }
+  if (root->type == CT_CLIP) {
+    double eps = 1e-9;
+    double A = root->a;
+    double B = root->b;
+    double C = root->c;
+    if (fabs(A) < eps) {
+      if (B * y + C < -eps) {
+        x0 = 1;
+        x1 = 0;
+      }
+    } else {
+      // X of intersection
+      double ix = - (B * y + C) / A;
+      if (A * x0 + B * y + C < eps) {
+        x0 = lround(fmax(x0, ix));
+      }
+      if (A * x1 + B * y + C < eps) {
+        x1 = lround(fmin(x1, ix));
+      }
+    }
+    if (x0 <= x1) {
+      event_list* start = malloc(sizeof(event_list));
+      if (!start) {
+        ImagingError_MemoryError();
+        return -1;
+      }
+      event_list* end = malloc(sizeof(event_list));
+      if (!end) {
+        free(start);
+        ImagingError_MemoryError();
+        return -1;
+      }
+      start->x = x0;
+      start->type = 1;
+      start->next = end;
+      end->x = x1;
+      end->type = -1;
+      end->next = NULL;
+      *ret = start;
+    } else {
+      *ret = NULL;
+    }
+    return 0;
+  }
+  if (root->type == CT_OR || root->type == CT_AND) {
+    event_list* l1;
+    event_list* l2;
+    if (clip_tree_do_clip(root->l, x0, y, x1, &l1) < 0) {
+      return -1;
+    }
+    if (clip_tree_do_clip(root->r, x0, y, x1, &l2) < 0) {
+      while (l1) {
+        l2 = l1->next;
+        free(l1);
+        l1 = l2;
+      }
+      return -1;
+    }
+    *ret = NULL;
+    event_list* tail = NULL;
+    int32_t k1 = 0;
+    int32_t k2 = 0;
+    while (l1 != NULL || l2 != NULL) {
+      event_list* t;
+      if (l2 == NULL || (l1 != NULL && (l1->x < l2->x || (l1->x == l2->x && l1->type > l2->type)))) {
+        t = l1;
+        k1 += t->type;
+        assert(k1 >= 0);
+        l1 = l1->next;
+      } else {
+        t = l2;
+        k2 += t->type;
+        assert(k2 >= 0);
+        l2 = l2->next;
+      }
+      t->next = NULL;
+      if ((root->type == CT_OR && (
+              (t->type == 1 && (tail == NULL || tail->type == -1)) ||
+              (t->type == -1 && k1 == 0 && k2 == 0)
+          )) || 
+          (root->type == CT_AND && (
+              (t->type == 1 && (tail == NULL || tail->type == -1) && k1 > 0 && k2 > 0) ||
+              (t->type == -1 && tail != NULL && tail->type == 1 && (k1 == 0 || k2 == 0))
+          ))) {
+        if (tail == NULL) {
+          *ret = t;
+        } else {
+          tail->next = t;
+        }
+        tail = t;
+      } else {
+        free(t);
+      }
+    }
+    return 0;
+  }
+  *ret = NULL;
+  return 0;
+}
+
+// One more layer of processing on top of the regular ellipse.
+// Uses the clipping tree.
+// Used for producing ellipse derivatives such as arc, chord, pie, etc.
+typedef struct {
+  ellipse_state st;
+  clip_node* root;
+  clip_node nodes[7];
+  int32_t node_count;
+  event_list* head;
+  int32_t y;
+} clip_ellipse_state;
+
+typedef void (*clip_ellipse_init)(clip_ellipse_state*, int32_t, int32_t, int32_t, float, float);
+
+void debug_clip_tree(clip_node* root, int space) {
+  if (root == NULL) {
+    return;
+  }
+  if (root->type == CT_CLIP) {
+    int t = space;
+    while (t--) {
+      fputc(' ', stderr);
+    }
+    fprintf(stderr, "clip %+fx%+fy%+f > 0\n", root->a, root->b, root->c);
+  } else {
+    debug_clip_tree(root->l, space + 2);
+    int t = space;
+    while (t--) {
+      fputc(' ', stderr);
+    }
+    fprintf(stderr, "%s\n", root->type == CT_AND ? "and" : "or");
+    debug_clip_tree(root->r, space + 2);
+  }
+  if (space == 0) {
+    fputc('\n', stderr);
+  }
+}
+
+// Resulting angles will satisfy 0 <= al < 360, al <= ar <= al + 360
+void normalize_angles(float* al, float* ar) {
+  if (*ar - *al >= 360) {
+    *al = 0;
+    *ar = 360;
+  } else {
+    *al = fmod(*al < 0 ? 360 - (fmod(-*al, 360)) : *al, 360);
+    *ar = *al + fmod(*ar < *al ? 360 - fmod(*al - *ar, 360) : *ar - *al, 360);
+  }
+}
+
+// An arc with caps orthogonal to the ellipse curve.
+void arc_init(clip_ellipse_state* s, int32_t a, int32_t b, int32_t w, float al, float ar) {
+  if (a < b) {
+    // transpose the coordinate system
+    arc_init(s, b, a, w, 90 - ar, 90 - al);
+    ellipse_init(&s->st, a, b, w);
+    clip_tree_transpose(s->root);
+  } else {
+    // a >= b, based on "wide" ellipse
+    ellipse_init(&s->st, a, b, w);
+
+    s->head = NULL;
+    s->node_count = 0;
+    normalize_angles(&al, &ar);
+
+    // building clipping tree, a lot of different cases
+    if (ar == al + 360) {
+      s->root = NULL;
+    } else {
+      clip_node* lc = s->nodes + s->node_count++;
+      clip_node* rc = s->nodes + s->node_count++;
+      lc->l = lc->r = rc->l = rc->r = NULL;
+      lc->type = rc->type = CT_CLIP;
+      lc->a = -a * sin(al * M_PI / 180.0);
+      lc->b = b * cos(al * M_PI / 180.0);
+      lc->c = (a * a - b * b) * sin(al * M_PI / 90.0) / 2.0;
+      rc->a = a * sin(ar * M_PI / 180.0);
+      rc->b = -b * cos(ar * M_PI / 180.0);
+      rc->c = (b * b - a * a) * sin(ar * M_PI / 90.0) / 2.0;
+      if (fmod(al, 180) == 0 || fmod(ar, 180) == 0) {
+        s->root = s->nodes + s->node_count++;
+        s->root->l = lc;
+        s->root->r = rc;
+        s->root->type = ar - al < 180 ? CT_AND : CT_OR;
+      } else if (((int)(al / 180) + (int)(ar / 180)) % 2 == 1) {
+        s->root = s->nodes + s->node_count++;
+        s->root->l = s->nodes + s->node_count++;
+        s->root->l->l = s->nodes + s->node_count++;
+        s->root->l->r = lc;
+        s->root->r = s->nodes + s->node_count++;
+        s->root->r->l = s->nodes + s->node_count++;
+        s->root->r->r = rc;
+        s->root->type = CT_OR;
+        s->root->l->type = CT_AND;
+        s->root->r->type = CT_AND;
+        s->root->l->l->type = CT_CLIP;
+        s->root->r->l->type = CT_CLIP;
+        s->root->l->l->l = s->root->l->l->r = NULL;
+        s->root->r->l->l = s->root->r->l->r = NULL;
+        s->root->l->l->a = s->root->l->l->c = 0;
+        s->root->r->l->a = s->root->r->l->c = 0;
+        s->root->l->l->b = (int)(al / 180) % 2 == 0 ? 1 : -1;
+        s->root->r->l->b = (int)(ar / 180) % 2 == 0 ? 1 : -1;
+      } else {
+        s->root = s->nodes + s->node_count++;
+        s->root->l = s->nodes + s->node_count++;
+        s->root->r = s->nodes + s->node_count++;
+        s->root->type = s->root->l->type = ar - al < 180 ? CT_AND : CT_OR;
+        s->root->l->l = lc;
+        s->root->l->r = rc;
+        s->root->r->type = CT_CLIP;
+        s->root->r->l = s->root->r->r = NULL;
+        s->root->r->a = s->root->r->c = 0;
+        s->root->r->b = ar < 180 || ar > 540 ? 1 : -1;
+      }
+    }
+  }
+}
+
+// A chord line.
+void chord_line_init(clip_ellipse_state* s, int32_t a, int32_t b, int32_t w, float al, float ar) {
+  ellipse_init(&s->st, a, b, a + b + 1);
+
+  s->head = NULL;
+  s->node_count = 0;
+
+  // line equation for chord
+  double xl = a * cos(al * M_PI / 180.0), xr = a * cos(ar * M_PI / 180.0);
+  double yl = b * sin(al * M_PI / 180.0), yr = b * sin(ar * M_PI / 180.0);
+  s->root = s->nodes + s->node_count++;
+  s->root->l = s->nodes + s->node_count++;
+  s->root->r = s->nodes + s->node_count++;
+  s->root->type = CT_AND;
+  s->root->l->type = s->root->r->type = CT_CLIP;
+  s->root->l->l = s->root->l->r = s->root->r->l = s->root->r->r = NULL;
+  s->root->l->a = yr - yl;
+  s->root->l->b = xl - xr;
+  s->root->l->c = -(s->root->l->a * xl + s->root->l->b * yl);
+  s->root->r->a = -s->root->l->a;
+  s->root->r->b = -s->root->l->b;
+  s->root->r->c = 2 * w * sqrt(pow(s->root->l->a, 2.0) + pow(s->root->l->b, 2.0)) - s->root->l->c;
+}
+
+// Pie side.
+void pie_side_init(clip_ellipse_state* s, int32_t a, int32_t b, int32_t w, float al, float _) {
+  ellipse_init(&s->st, a, b, a + b + 1);
+
+  s->head = NULL;
+  s->node_count = 0;
+
+  double xl = a * cos(al * M_PI / 180.0);
+  double yl = b * sin(al * M_PI / 180.0);
+  double a1 = -yl;
+  double b1 = xl;
+  double c1 = w * sqrt(a1 * a1 + b1 * b1);
+
+  s->root = s->nodes + s->node_count++;
+  s->root->type = CT_AND;
+  s->root->l = s->nodes + s->node_count++;
+  s->root->l->type = CT_AND;
+
+  clip_node* cnode;
+  cnode = s->nodes + s->node_count++;
+  cnode->l = cnode->r = NULL;
+  cnode->type = CT_CLIP;
+  cnode->a = a1;
+  cnode->b = b1;
+  cnode->c = c1;
+  s->root->l->l = cnode;
+  cnode = s->nodes + s->node_count++;
+  cnode->l = cnode->r = NULL;
+  cnode->type = CT_CLIP;
+  cnode->a = -a1;
+  cnode->b = -b1;
+  cnode->c = c1;
+  s->root->l->r = cnode;
+  cnode = s->nodes + s->node_count++;
+  cnode->l = cnode->r = NULL;
+  cnode->type = CT_CLIP;
+  cnode->a = b1;
+  cnode->b = -a1;
+  cnode->c = 0;
+  s->root->r = cnode;
+}
+
+// A chord.
+void chord_init(clip_ellipse_state* s, int32_t a, int32_t b, int32_t w, float al, float ar) {
+  ellipse_init(&s->st, a, b, w);
+
+  s->head = NULL;
+  s->node_count = 0;
+
+  // line equation for chord
+  double xl = a * cos(al * M_PI / 180.0), xr = a * cos(ar * M_PI / 180.0);
+  double yl = b * sin(al * M_PI / 180.0), yr = b * sin(ar * M_PI / 180.0);
+  s->root = s->nodes + s->node_count++;
+  s->root->l = s->root->r = NULL;
+  s->root->type = CT_CLIP;
+  s->root->a = yr - yl;
+  s->root->b = xl - xr;
+  s->root->c = -(s->root->a * xl + s->root->b * yl);
+}
+
+// A pie. Can also be used to draw an arc with ugly sharp caps.
+void pie_init(clip_ellipse_state* s, int32_t a, int32_t b, int32_t w, float al, float ar) {
+  ellipse_init(&s->st, a, b, w);
+
+  s->head = NULL;
+  s->node_count = 0;
+
+  // line equations for pie sides
+  double xl = a * cos(al * M_PI / 180.0), xr = a * cos(ar * M_PI / 180.0);
+  double yl = b * sin(al * M_PI / 180.0), yr = b * sin(ar * M_PI / 180.0);
+
+  clip_node* lc = s->nodes + s->node_count++;
+  clip_node* rc = s->nodes + s->node_count++;
+  lc->l = lc->r = rc->l = rc->r = NULL;
+  lc->type = rc->type = CT_CLIP;
+  lc->a = -yl;
+  lc->b = xl;
+  lc->c = 0;
+  rc->a = yr;
+  rc->b = -xr;
+  rc->c = 0;
+        
+  s->root = s->nodes + s->node_count++;
+  s->root->l = lc;
+  s->root->r = rc;
+  s->root->type = ar - al < 180 ? CT_AND : CT_OR;
+}
+
+void clip_ellipse_free(clip_ellipse_state* s) {
+  while (s->head != NULL) {
+    event_list* t = s->head;
+    s->head = s->head->next;
+    free(t);
+  }
+}
+
+int8_t clip_ellipse_next(clip_ellipse_state* s, int32_t* ret_x0, int32_t* ret_y, int32_t* ret_x1) {
+  int32_t x0, y, x1;
+  while (s->head == NULL && ellipse_next(&s->st, &x0, &y, &x1) >= 0) {
+    if (clip_tree_do_clip(s->root, x0, y, x1, &s->head) < 0) {
+      return -2;
+    }
+    s->y = y;
+  }
+  if (s->head != NULL) {
+    *ret_y = s->y;
+    event_list* t = s->head;
+    s->head = s->head->next;
+    *ret_x0 = t->x;
+    free(t);
+    t = s->head;
+    assert(t != NULL);
+    s->head = s->head->next;
+    *ret_x1 = t->x;
+    free(t);
+    return 0;
+  }
+  return -1;
+}
+
+static int
+ellipseNew(Imaging im, int x0, int y0, int x1, int y1,
+        const void* ink_, int fill,
+        int width, int op)
 {
-    return ellipse(im, x0, y0, x1, y1, start, end, ink, fill, width, CHORD, op);
+    DRAW* draw;
+    INT32 ink;
+    DRAWINIT();
+
+    int a = x1 - x0;
+    int b = y1 - y0;
+    if (a < 0 || b < 0) {
+      return 0;
+    }
+    if (fill) {
+      width = a + b;
+    }
+
+    ellipse_state st;
+    ellipse_init(&st, a, b, width);
+    int32_t X0, Y, X1;
+    while (ellipse_next(&st, &X0, &Y, &X1) != -1) {
+      draw->hline(im, x0 + (X0 + a) / 2, y0 + (Y + b) / 2, x0 + (X1 + a) / 2, ink);
+    }
+    return 0;
+}
+
+static int
+clipEllipseNew(Imaging im, int x0, int y0, int x1, int y1,
+        float start, float end,
+        const void* ink_, int width, int op, clip_ellipse_init init)
+{
+    DRAW* draw;
+    INT32 ink;
+    DRAWINIT();
+
+    int a = x1 - x0;
+    int b = y1 - y0;
+    if (a < 0 || b < 0) {
+      return 0;
+    }
+
+    clip_ellipse_state st;
+    init(&st, a, b, width, start, end);
+    // debug_clip_tree(st.root, 0);
+    int32_t X0, Y, X1;
+    int next_code;
+    while ((next_code = clip_ellipse_next(&st, &X0, &Y, &X1)) >= 0) {
+      draw->hline(im, x0 + (X0 + a) / 2, y0 + (Y + b) / 2, x0 + (X1 + a) / 2, ink);
+    }
+    clip_ellipse_free(&st);
+    return next_code == -1 ? 0 : -1;
+}
+static int
+arcNew(Imaging im, int x0, int y0, int x1, int y1,
+        float start, float end,
+        const void* ink_, int width, int op)
+{
+  return clipEllipseNew(im, x0, y0, x1, y1, start, end, ink_, width, op, arc_init);
+}
+
+static int
+chordNew(Imaging im, int x0, int y0, int x1, int y1,
+        float start, float end,
+        const void* ink_, int width, int op)
+{
+  return clipEllipseNew(im, x0, y0, x1, y1, start, end, ink_, width, op, chord_init);
+}
+
+static int
+chordLineNew(Imaging im, int x0, int y0, int x1, int y1,
+        float start, float end,
+        const void* ink_, int width, int op)
+{
+  return clipEllipseNew(im, x0, y0, x1, y1, start, end, ink_, width, op, chord_line_init);
+}
+
+static int
+pieNew(Imaging im, int x0, int y0, int x1, int y1,
+        float start, float end,
+        const void* ink_, int width, int op)
+{
+  return clipEllipseNew(im, x0, y0, x1, y1, start, end, ink_, width, op, pie_init);
+}
+
+static int
+pieSideNew(Imaging im, int x0, int y0, int x1, int y1,
+        float start,
+        const void* ink_, int width, int op)
+{
+  return clipEllipseNew(im, x0, y0, x1, y1, start, 0, ink_, width, op, pie_side_init);
 }
 
 int
 ImagingDrawEllipse(Imaging im, int x0, int y0, int x1, int y1,
                    const void* ink, int fill, int width, int op)
 {
-    return ellipse(im, x0, y0, x1, y1, 0, 360, ink, fill, width, CHORD, op);
+    return ellipseNew(im, x0, y0, x1, y1, ink, fill, width, op);
 }
 
+int
+ImagingDrawArc(Imaging im, int x0, int y0, int x1, int y1,
+               float start, float end, const void* ink, int width, int op)
+{
+    normalize_angles(&start, &end);
+    if (start + 360 == end) {
+      return ImagingDrawEllipse(im, x0, y0, x1, y1, ink, 0, width, op);
+    }
+    if (start == end) {
+      return 0;
+    }
+    return arcNew(im, x0, y0, x1, y1, start, end, ink, width, op);
+}
+
+
+int
+ImagingDrawChord(Imaging im, int x0, int y0, int x1, int y1,
+                 float start, float end, const void* ink, int fill,
+                 int width, int op)
+{
+    normalize_angles(&start, &end);
+    if (start + 360 == end) {
+      return ImagingDrawEllipse(im, x0, y0, x1, y1, ink, fill, width, op);
+    }
+    if (start == end) {
+      return 0;
+    }
+    if (fill) {
+      return chordNew(im, x0, y0, x1, y1, start, end, ink, x1 - x0 + y1 - y0 + 1, op);
+    } else {
+      if (chordLineNew(im, x0, y0, x1, y1, start, end, ink, width, op)) {
+        return -1;
+      }
+      return chordNew(im, x0, y0, x1, y1, start, end, ink, width, op);
+    }
+}
+
+
 int
 ImagingDrawPieslice(Imaging im, int x0, int y0, int x1, int y1,
                     float start, float end, const void* ink, int fill,
                     int width, int op)
 {
-    return ellipse(im, x0, y0, x1, y1, start, end, ink, fill, width, PIESLICE, op);
+    normalize_angles(&start, &end);
+    if (start + 360 == end) {
+      return ellipseNew(im, x0, y0, x1, y1, ink, fill, width, op);
+    }
+    if (start == end) {
+      return 0;
+    }
+    if (fill) {
+      return pieNew(im, x0, y0, x1, y1, start, end, ink, x1 + y1 - x0 - y0, op);
+    } else {
+      if (pieSideNew(im, x0, y0, x1, y1, start, ink, width, op)) {
+        return -1;
+      }
+      if (pieSideNew(im, x0, y0, x1, y1, end, ink, width, op)) {
+        return -1;
+      }
+      int xc = lround((x0 + x1 - width) / 2.0), yc = lround((y0 + y1 - width) / 2.0);
+      ellipseNew(im, xc, yc, xc + width - 1, yc + width - 1, ink, 1, 0, op);
+      return pieNew(im, x0, y0, x1, y1, start, end, ink, width, op);
+    }
 }
 
+
 /* -------------------------------------------------------------------- */
 
 /* experimental level 2 ("arrow") graphics stuff.  this implements
@@ -1026,8 +1591,9 @@ ImagingOutlineNew(void)
     ImagingOutline outline;
 
     outline = calloc(1, sizeof(struct ImagingOutlineInstance));
-    if (!outline)
+    if (!outline) {
         return (ImagingOutline) ImagingError_MemoryError();
+    }
 
     outline->edges = NULL;
     outline->count = outline->size = 0;
@@ -1040,11 +1606,13 @@ ImagingOutlineNew(void)
 void
 ImagingOutlineDelete(ImagingOutline outline)
 {
-    if (!outline)
+    if (!outline) {
         return;
+    }
 
-    if (outline->edges)
+    if (outline->edges) {
         free(outline->edges);
+    }
 
     free(outline);
 }
@@ -1068,8 +1636,9 @@ allocate(ImagingOutline outline, int extra)
             /* malloc check ok, overflow checked above */
             e = realloc(outline->edges, outline->size * sizeof(Edge));
         }
-        if (!e)
+        if (!e) {
             return NULL;
+        }
         outline->edges = e;
     }
 
@@ -1095,8 +1664,9 @@ ImagingOutlineLine(ImagingOutline outline, float x1, float y1)
     Edge* e;
 
     e = allocate(outline, 1);
-    if (!e)
+    if (!e) {
         return -1; /* out of memory */
+    }
 
     add_edge(e, (int) outline->x, (int) outline->y, (int) x1, (int) y1);
 
@@ -1117,8 +1687,9 @@ ImagingOutlineCurve(ImagingOutline outline, float x1, float y1,
 #define STEPS 32
 
     e = allocate(outline, STEPS);
-    if (!e)
+    if (!e) {
         return -1; /* out of memory */
+    }
 
     xo = outline->x;
     yo = outline->y;
@@ -1153,8 +1724,9 @@ ImagingOutlineCurve(ImagingOutline outline, float x1, float y1,
 int
 ImagingOutlineClose(ImagingOutline outline)
 {
-    if (outline->x == outline->x0 && outline->y == outline->y0)
+    if (outline->x == outline->x0 && outline->y == outline->y0) {
         return 0;
+    }
     return ImagingOutlineLine(outline, outline->x0, outline->y0);
 }
 
@@ -1191,14 +1763,16 @@ ImagingOutlineTransform(ImagingOutline outline, double a[6])
         y0 = eIn->y0;
 
         /* FIXME: ouch! */
-        if (eIn->x0 == eIn->xmin)
+        if (eIn->x0 == eIn->xmin) {
             x1 = eIn->xmax;
-        else
+        } else {
             x1 = eIn->xmin;
-        if (eIn->y0 == eIn->ymin)
+        }
+        if (eIn->y0 == eIn->ymin) {
             y1 = eIn->ymax;
-        else
+        } else {
             y1 = eIn->ymin;
+        }
 
         /* full moon tonight!  if this doesn't work, you may need to
            upgrade your compiler (make sure you have the right service
