@@ -27,12 +27,10 @@
 
 
 #include "Python.h"
-#include "Imaging.h"
+#include "libImaging/Imaging.h"
 
 #include <math.h>
 
-#include "py3.h"
-
 /* compatibility wrappers (defined in _imaging.c) */
 extern int PyImaging_CheckBuffer(PyObject* buffer);
 extern int PyImaging_GetBuffer(PyObject* buffer, Py_buffer *view);
@@ -58,13 +56,14 @@ alloc_array(Py_ssize_t count)
         PyErr_NoMemory();
         return NULL;
     }
-    if (count > (SIZE_MAX / (2 * sizeof(double))) - 1 ) {
+    if ((unsigned long long)count > (SIZE_MAX / (2 * sizeof(double))) - 1 ) {
         PyErr_NoMemory();
         return NULL;
     }
     xy = malloc(2 * count * sizeof(double) + 1);
-    if (!xy)
+    if (!xy) {
         PyErr_NoMemory();
+    }
     return xy;
 }
 
@@ -76,18 +75,23 @@ path_new(Py_ssize_t count, double* xy, int duplicate)
     if (duplicate) {
         /* duplicate path */
         double* p = alloc_array(count);
-        if (!p)
+        if (!p) {
             return NULL;
+        }
         memcpy(p, xy, count * 2 * sizeof(double));
         xy = p;
     }
 
-    if (PyType_Ready(&PyPathType) < 0)
+    if (PyType_Ready(&PyPathType) < 0) {
+        free(xy);
         return NULL;
+    }
 
     path = PyObject_New(PyPathObject, &PyPathType);
-    if (path == NULL)
+    if (path == NULL) {
+        free(xy);
         return NULL;
+    }
 
     path->count = count;
     path->xy = xy;
@@ -118,8 +122,9 @@ PyPath_Flatten(PyObject* data, double **pxy)
         /* This was another path object. */
         PyPathObject *path = (PyPathObject*) data;
         xy = alloc_array(path->count);
-        if (!xy)
+        if (!xy) {
             return -1;
+        }
         memcpy(xy, path->xy, 2 * path->count * sizeof(double));
         *pxy = xy;
         return path->count;
@@ -129,13 +134,15 @@ PyPath_Flatten(PyObject* data, double **pxy)
         /* Assume the buffer contains floats */
         Py_buffer buffer;
         if (PyImaging_GetBuffer(data, &buffer) == 0) {
-            int n = buffer.len / (2 * sizeof(float));
             float *ptr = (float*) buffer.buf;
+            n = buffer.len / (2 * sizeof(float));
             xy = alloc_array(n);
-            if (!xy)
+            if (!xy) {
                 return -1;
-            for (i = 0; i < n+n; i++)
+            }
+            for (i = 0; i < n+n; i++) {
                 xy[i] = ptr[i];
+            }
             *pxy = xy;
             PyBuffer_Release(&buffer);
             return n;
@@ -151,26 +158,28 @@ PyPath_Flatten(PyObject* data, double **pxy)
     j = 0;
     n = PyObject_Length(data);
     /* Just in case __len__ breaks (or doesn't exist) */
-    if (PyErr_Occurred())
+    if (PyErr_Occurred()) {
         return -1;
+    }
 
     /* Allocate for worst case */
     xy = alloc_array(n);
-    if (!xy)
+    if (!xy) {
         return -1;
+    }
 
     /* Copy table to path array */
     if (PyList_Check(data)) {
         for (i = 0; i < n; i++) {
             double x, y;
             PyObject *op = PyList_GET_ITEM(data, i);
-            if (PyFloat_Check(op))
+            if (PyFloat_Check(op)) {
                 xy[j++] = PyFloat_AS_DOUBLE(op);
-            else if (PyInt_Check(op))
-                xy[j++] = (float) PyInt_AS_LONG(op);
-            else if (PyNumber_Check(op))
+            } else if (PyLong_Check(op)) {
+                xy[j++] = (float) PyLong_AS_LONG(op);
+            } else if (PyNumber_Check(op)) {
                 xy[j++] = PyFloat_AsDouble(op);
-            else if (PyArg_ParseTuple(op, "dd", &x, &y)) {
+            } else if (PyArg_ParseTuple(op, "dd", &x, &y)) {
                 xy[j++] = x;
                 xy[j++] = y;
             } else {
@@ -182,13 +191,13 @@ PyPath_Flatten(PyObject* data, double **pxy)
         for (i = 0; i < n; i++) {
             double x, y;
             PyObject *op = PyTuple_GET_ITEM(data, i);
-            if (PyFloat_Check(op))
+            if (PyFloat_Check(op)) {
                 xy[j++] = PyFloat_AS_DOUBLE(op);
-            else if (PyInt_Check(op))
-                xy[j++] = (float) PyInt_AS_LONG(op);
-            else if (PyNumber_Check(op))
+            } else if (PyLong_Check(op)) {
+                xy[j++] = (float) PyLong_AS_LONG(op);
+            } else if (PyNumber_Check(op)) {
                 xy[j++] = PyFloat_AsDouble(op);
-            else if (PyArg_ParseTuple(op, "dd", &x, &y)) {
+            } else if (PyArg_ParseTuple(op, "dd", &x, &y)) {
                 xy[j++] = x;
                 xy[j++] = y;
             } else {
@@ -211,13 +220,13 @@ PyPath_Flatten(PyObject* data, double **pxy)
                     return -1;
                 }
             }
-            if (PyFloat_Check(op))
+            if (PyFloat_Check(op)) {
                 xy[j++] = PyFloat_AS_DOUBLE(op);
-            else if (PyInt_Check(op))
-                xy[j++] = (float) PyInt_AS_LONG(op);
-            else if (PyNumber_Check(op))
+            } else if (PyLong_Check(op)) {
+                xy[j++] = (float) PyLong_AS_LONG(op);
+            } else if (PyNumber_Check(op)) {
                 xy[j++] = PyFloat_AsDouble(op);
-            else if (PyArg_ParseTuple(op, "dd", &x, &y)) {
+            } else if (PyArg_ParseTuple(op, "dd", &x, &y)) {
                 xy[j++] = x;
                 xy[j++] = y;
             } else {
@@ -255,19 +264,22 @@ PyPath_Create(PyObject* self, PyObject* args)
 
         /* number of vertices */
         xy = alloc_array(count);
-        if (!xy)
+        if (!xy) {
             return NULL;
+        }
 
     } else {
 
         /* sequence or other path */
         PyErr_Clear();
-        if (!PyArg_ParseTuple(args, "O", &data))
+        if (!PyArg_ParseTuple(args, "O", &data)) {
             return NULL;
+        }
 
         count = PyPath_Flatten(data, &xy);
-        if (count < 0)
+        if (count < 0) {
             return NULL;
+        }
     }
 
     return (PyObject*) path_new(count, xy, 0);
@@ -289,8 +301,9 @@ path_compact(PyPathObject* self, PyObject* args)
 
     double cityblock = 2.0;
 
-    if (!PyArg_ParseTuple(args, "|d:compact", &cityblock))
+    if (!PyArg_ParseTuple(args, "|d:compact", &cityblock)) {
         return NULL;
+    }
 
     xy = self->xy;
 
@@ -321,8 +334,9 @@ path_getbbox(PyPathObject* self, PyObject* args)
     double *xy;
     double x0, y0, x1, y1;
 
-    if (!PyArg_ParseTuple(args, ":getbbox"))
+    if (!PyArg_ParseTuple(args, ":getbbox")) {
         return NULL;
+    }
 
     xy = self->xy;
 
@@ -330,14 +344,18 @@ path_getbbox(PyPathObject* self, PyObject* args)
     y0 = y1 = xy[1];
 
     for (i = 1; i < self->count; i++) {
-        if (xy[i+i] < x0)
+        if (xy[i+i] < x0) {
             x0 = xy[i+i];
-        if (xy[i+i] > x1)
+        }
+        if (xy[i+i] > x1) {
             x1 = xy[i+i];
-        if (xy[i+i+1] < y0)
+        }
+        if (xy[i+i+1] < y0) {
             y0 = xy[i+i+1];
-        if (xy[i+i+1] > y1)
+        }
+        if (xy[i+i+1] > y1) {
             y1 = xy[i+i+1];
+        }
     }
 
     return Py_BuildValue("dddd", x0, y0, x1, y1);
@@ -346,8 +364,9 @@ path_getbbox(PyPathObject* self, PyObject* args)
 static PyObject*
 path_getitem(PyPathObject* self, Py_ssize_t i)
 {
-    if (i < 0)
+    if (i < 0) {
         i = self->count + i;
+    }
     if (i < 0 || i >= self->count) {
         PyErr_SetString(PyExc_IndexError, "path index out of range");
         return NULL;
@@ -360,16 +379,19 @@ static PyObject*
 path_getslice(PyPathObject* self, Py_ssize_t ilow, Py_ssize_t ihigh)
 {
     /* adjust arguments */
-    if (ilow < 0)
+    if (ilow < 0) {
         ilow = 0;
-    else if (ilow >= self->count)
+    } else if (ilow >= self->count) {
         ilow = self->count;
-    if (ihigh < 0)
+    }
+    if (ihigh < 0) {
         ihigh = 0;
-    if (ihigh < ilow)
+    }
+    if (ihigh < ilow) {
         ihigh = ilow;
-    else if (ihigh > self->count)
+    } else if (ihigh > self->count) {
         ihigh = self->count;
+    }
 
     return (PyObject*) path_new(ihigh - ilow, self->xy + ilow * 2, 1);
 }
@@ -388,8 +410,9 @@ path_map(PyPathObject* self, PyObject* args)
     double *xy;
     PyObject* function;
 
-    if (!PyArg_ParseTuple(args, "O:map", &function))
+    if (!PyArg_ParseTuple(args, "O:map", &function)) {
         return NULL;
+    }
 
     xy = self->xy;
 
@@ -430,8 +453,9 @@ path_setitem(PyPathObject* self, Py_ssize_t i, PyObject* op)
 
     xy = &self->xy[i+i];
 
-    if (!PyArg_ParseTuple(op, "dd", &xy[0], &xy[1]))
+    if (!PyArg_ParseTuple(op, "dd", &xy[0], &xy[1])) {
         return -1;
+    }
 
     return 0;
 }
@@ -443,16 +467,18 @@ path_tolist(PyPathObject* self, PyObject* args)
     Py_ssize_t i;
 
     int flat = 0;
-    if (!PyArg_ParseTuple(args, "|i:tolist", &flat))
+    if (!PyArg_ParseTuple(args, "|i:tolist", &flat)) {
         return NULL;
+    }
 
     if (flat) {
         list = PyList_New(self->count*2);
         for (i = 0; i < self->count*2; i++) {
             PyObject* item;
             item = PyFloat_FromDouble(self->xy[i]);
-            if (!item)
+            if (!item) {
                 goto error;
+            }
             PyList_SetItem(list, i, item);
         }
     } else {
@@ -460,8 +486,9 @@ path_tolist(PyPathObject* self, PyObject* args)
         for (i = 0; i < self->count; i++) {
             PyObject* item;
             item = Py_BuildValue("dd", self->xy[i+i], self->xy[i+i+1]);
-            if (!item)
+            if (!item) {
                 goto error;
+            }
             PyList_SetItem(list, i, item);
         }
     }
@@ -485,19 +512,20 @@ path_transform(PyPathObject* self, PyObject* args)
 
     if (!PyArg_ParseTuple(args, "(dddddd)|d:transform",
                           &a, &b, &c, &d, &e, &f,
-                          &wrap))
+                          &wrap)) {
         return NULL;
+    }
 
     xy = self->xy;
 
     /* transform the coordinate set */
-    if (b == 0.0 && d == 0.0)
+    if (b == 0.0 && d == 0.0) {
         /* scaling */
         for (i = 0; i < self->count; i++) {
             xy[i+i]   = a*xy[i+i]+c;
             xy[i+i+1] = e*xy[i+i+1]+f;
         }
-    else
+    } else {
         /* affine transform */
         for (i = 0; i < self->count; i++) {
             double x = xy[i+i];
@@ -505,11 +533,14 @@ path_transform(PyPathObject* self, PyObject* args)
             xy[i+i]   = a*x+b*y+c;
             xy[i+i+1] = d*x+e*y+f;
         }
+    }
 
     /* special treatment of geographical map data */
-    if (wrap != 0.0)
-        for (i = 0; i < self->count; i++)
+    if (wrap != 0.0) {
+        for (i = 0; i < self->count; i++) {
             xy[i+i] = fmod(xy[i+i], wrap);
+        }
+    }
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -540,21 +571,18 @@ path_subscript(PyPathObject* self, PyObject* item) {
     if (PyIndex_Check(item)) {
         Py_ssize_t i;
         i = PyNumber_AsSsize_t(item, PyExc_IndexError);
-        if (i == -1 && PyErr_Occurred())
+        if (i == -1 && PyErr_Occurred()) {
             return NULL;
+        }
         return path_getitem(self, i);
     }
     if (PySlice_Check(item)) {
         int len = 4;
         Py_ssize_t start, stop, step, slicelength;
 
-#if PY_VERSION_HEX >= 0x03020000
-        if (PySlice_GetIndicesEx(item, len, &start, &stop, &step, &slicelength) < 0)
+        if (PySlice_GetIndicesEx(item, len, &start, &stop, &step, &slicelength) < 0) {
             return NULL;
-#else
-        if (PySlice_GetIndicesEx((PySliceObject*)item, len, &start, &stop, &step, &slicelength) < 0)
-            return NULL;
-#endif
+        }
 
         if (slicelength <= 0) {
             double *xy = alloc_array(0);
